#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_NeoPixel.h>
#include "pitches.h"

// OLED display size
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define OLED_RESET -1 // Reset pin (not used with I2C)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

// NeoPixel setup
#define NEOPIXEL_PIN 6
#define NUM_PIXELS 8
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_PIXELS, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);

// Input pins
#define BUTTON_PIN 8
#define CLEAR_BUTTON_PIN 5
#define SPEAKER_PIN 7

//Timer durations (seconds)
// const int WORK_TIME = 10;  // 25 minutes working
// const int SHORT_BREAK = 5 * 60; // 5 minute short break
// const int LONG_BREAK = 15 * 60; // 15 minute long break

const int WORK_TIME = 25 * 60;  // 25 minutes working
const int SHORT_BREAK = 5 * 60; // 5 minute short break
const int LONG_BREAK = 15 * 60; // 15 minute long break


// Melody arrays for The Witcher Theme
int melody[] = {
  NOTE_E4, NOTE_G4, NOTE_B4, NOTE_G4, NOTE_E4, NOTE_B4, NOTE_G4, NOTE_E4,
  NOTE_A4, NOTE_E4, NOTE_A4, NOTE_E4, NOTE_B4, NOTE_G4, NOTE_B4, NOTE_E4,
  NOTE_E4, NOTE_G4, NOTE_B4, NOTE_E4, NOTE_B4, NOTE_G4, NOTE_E4, NOTE_B4,
  NOTE_G4, NOTE_E4, NOTE_A4, NOTE_E4, NOTE_A4, NOTE_E4, NOTE_B4
};

int durations[] = {
  8, 8, 8, 8, 4, 8, 8, 8,
  8, 8, 8, 8, 8, 8, 8, 8,
  8, 8, 8, 8, 8, 8, 8, 8,
  8, 8, 8, 8, 8, 8, 4
};

int melodyLength = sizeof(melody) / sizeof(melody[0]);

// State variables
int timerState = 0;         // 0: Stopped, 1: Running, 2: Paused
int currentPhase = 0;       // 0: Work, 1: Short Break, 2: Long Break
unsigned long startTime = 0;
unsigned long elapsedTime = 0;
unsigned long phaseStartTime = 0;
unsigned long lastUpdateTime = 0;
unsigned long blinkStartTime = 0;
int blinkCount = 0;
bool ledState = false;      // Track if LEDs are on or off

// Temporary counters
unsigned long workTimeCounter = 0;
unsigned long shortBreakCounter = 0;
unsigned long longBreakCounter = 0;

// Function prototypes
void showMessage(const char *message);
void handleButtonPress();
void startTimer();
void updateTimer();
void phaseCompleted();
void updateCounters();
void startPhase(int duration, uint32_t color);
void nextPhase();
void resetTimer();
void updateDisplay(int seconds);
void blinkLEDs();
void showElapsedTime();
void clearCounters();
int getRemainingTime();
void playMelody();

void setup() {
  Serial.begin(9600); // Initialize serial communication at 9600 baud rate
  Serial.println("Debugging nextPhase()");

  pinMode(BUTTON_PIN, INPUT_PULLUP);
  pinMode(CLEAR_BUTTON_PIN, INPUT_PULLUP);
  pinMode(SPEAKER_PIN, OUTPUT);

  // Initialize OLED display
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
    while (true); // Halt if display initialization fails
  }
  display.clearDisplay();
  display.display();

  // Initialize NeoPixel
  strip.begin();
  strip.setBrightness(10); // Set brightness (adjustable) (was 50 and too bright)
  strip.show(); // Turn off all pixels initially

  // Start with the Work phase (Green LEDs)
  startPhase(WORK_TIME, strip.Color(0, 255, 0)); // Green for Work

  // Show initial message
  showMessage("Pomodoro Timer\nPress Button to Start");
}

void loop() {
  if (digitalRead(BUTTON_PIN) == LOW) {
    delay(200); // Debounce
    handleButtonPress();
  }

  if (digitalRead(CLEAR_BUTTON_PIN) == LOW) {
    delay(200); // Debounce
    clearCounters();
    resetTimer();
  }

  if (timerState == 1) {
    unsigned long currentMillis = millis();
    if (currentMillis - lastUpdateTime >= 1000) { // Update every second (1000 ms)
      lastUpdateTime = currentMillis;
      updateTimer();
    }

    if (blinkCount > 0 && currentMillis - blinkStartTime >= 500) { // Flash every 500ms
      blinkLEDs();
    }
  }
}

void showMessage(const char *message) {
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 0);
  display.println(message);
  display.display();
}
void handleButtonPress() {
  if (timerState == 0) { // Start
    timerState = 1;
    phaseStartTime = millis(); 
    elapsedTime = 0;  // Reset elapsed time
    startTimer();
  } else if (timerState == 1) { // Pause
    timerState = 2;
    elapsedTime += millis() - phaseStartTime;
    showElapsedTime();
  } else if (timerState == 2) { // Resume
    timerState = 1;
    phaseStartTime = millis() - elapsedTime; // Resume from paused time
    startTimer();
  }
}

void startTimer() {
  phaseStartTime = millis() - elapsedTime;  // Adjust for elapsed time
  lastUpdateTime = millis();  
  updateDisplay(getRemainingTime());
}

void updateTimer() {
  int remainingTime = getRemainingTime();
  if (remainingTime <= 0) {
    phaseCompleted();
  } else {
    updateDisplay(remainingTime);
  }
}

void phaseCompleted() {
  blinkCount = 2;  // Number of blinks
  blinkStartTime = millis();
  updateCounters();
  playMelody();
  showMessage("Phase Completed\nNext Phase Starting...");

  // Trigger LED blinking before transitioning
  delay(1000); // Brief pause before transitioning
  nextPhase();
}

void updateCounters() {
  unsigned long phaseTime = elapsedTime + (millis() - phaseStartTime);
  if (currentPhase == 0) {
    workTimeCounter += phaseTime;
  } else if (currentPhase == 1) {
    shortBreakCounter += phaseTime;
  } else if (currentPhase == 2) {
    longBreakCounter += phaseTime;
  }
}

void startPhase(int duration, uint32_t color) {
  strip.clear();
  for (int i = 0; i < NUM_PIXELS; i++) {
    strip.setPixelColor(i, color);
  }
  strip.show();

  phaseStartTime = millis();
  elapsedTime = 0;
  lastUpdateTime = millis();
  updateDisplay(duration);
}

void nextPhase() {
  if (currentPhase == 0) {
    currentPhase = 1;
  } else if (currentPhase == 1) {
    currentPhase = 2;
  } else {
    currentPhase = 0;
  }

  int duration = (currentPhase == 0) ? WORK_TIME : (currentPhase == 1) ? SHORT_BREAK : LONG_BREAK;
  uint32_t color = (currentPhase == 0) ? strip.Color(0, 255, 0) : 
                   (currentPhase == 1) ? strip.Color(0, 0, 255) : strip.Color(255, 0, 0);

  startPhase(duration, color);
  timerState = 1; // Ensure the timer starts
  startTimer();   // Explicitly start the countdown
}

void resetTimer() {
  strip.clear();
  strip.show();
  timerState = 0;
  currentPhase = 0;
  workTimeCounter = 0;
  shortBreakCounter = 0;
  longBreakCounter = 0;

   // Start with the Work phase (Green LEDs)
  startPhase(WORK_TIME, strip.Color(0, 255, 0)); // Green for Work


  showMessage("Timer Reset\nPress Button to Start");
}

void updateDisplay(int seconds) {
  int minutes = seconds / 60;
  int remainingSeconds = seconds % 60;

  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 0);
  display.println("Pomodoro Timer");
  display.setCursor(0, 20);
  display.print("Time: ");
  display.print(minutes);
  display.print(":");
  if (remainingSeconds < 10) display.print("0");
  display.print(remainingSeconds);
  display.display();
}

void blinkLEDs() {
  if (ledState) {
    strip.clear();
    ledState = false;
  } else {
    for (int i = 0; i < NUM_PIXELS; i++) {
      strip.setPixelColor(i, strip.Color(255, 255, 255));
    }
    ledState = true;
  }

  strip.show();
  blinkCount--;

  if (blinkCount <= 0) {
    uint32_t color;
    if (currentPhase == 0) {
      color = strip.Color(0, 255, 0);
    } else if (currentPhase == 1) {
      color = strip.Color(0, 0, 255);
    } else {
      color = strip.Color(255, 0, 0);
    }
    for (int i = 0; i < NUM_PIXELS; i++) {
      strip.setPixelColor(i, color);
    }
    strip.show();
  }
}

  void showElapsedTime() {
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(SSD1306_WHITE);
    display.setCursor(0, 0);
    display.println("Elapsed Time:");
    display.setCursor(0, 20);
    display.print("W:");
    display.print(workTimeCounter / 60000);
    display.print(" S:");
    display.print(shortBreakCounter / 60000);
    display.print(" L:");
    display.print(longBreakCounter / 60000);
    display.display();
  }

void clearCounters() {
  workTimeCounter = 0;
  shortBreakCounter = 0;
  longBreakCounter = 0;
  showElapsedTime();
}

int getRemainingTime() {
  int phaseDuration = (currentPhase == 0) ? WORK_TIME : (currentPhase == 1) ? SHORT_BREAK : LONG_BREAK;

  unsigned long elapsedMillis;
  if (timerState == 2) { // If paused
    elapsedMillis = elapsedTime;
  } else {
    elapsedMillis = elapsedTime + (millis() - phaseStartTime);
  }

  unsigned long totalElapsedSeconds = elapsedMillis / 1000;
  return max(0, phaseDuration - totalElapsedSeconds);
}

 

void playMelody() {
  int tempo = 120; // Set the tempo in beats per minute (BPM)
  int quarterNoteDuration = 60000 / tempo; // Duration of a quarter note in milliseconds
  
  for (int i = 0; i < melodyLength; i++) {
    int noteDuration = quarterNoteDuration / durations[i]; // Calculate duration of note
    tone(SPEAKER_PIN, melody[i], noteDuration); // Play the note
    delay(noteDuration * 1.3); // Slightly longer delay to avoid overlapping notes
    noTone(SPEAKER_PIN); // Stop the note to prepare for the next one
    delay(50); // Small delay to ensure there's a gap between notes
  }
}